# frozen_string_literal: true

Application.boot(:clients, namespace: true) do
  init do
    require 'cht-lib-fabric/helpers/graphql/http'
    require 'cht-lib-fabric/helpers/graphql/shard_assignment_lookup'
    require 'cht-lib-fabric/services'
    require 'cht-lib-fabric/clients'
    require 'user_management/clients/role_management'
    require 'user_management/clients/privilege_escalation_checker'
  end

  start do |application|
    use :settings

    sa_client = ::Platform::Helpers::GraphQL::ShardAssignmentLookup.new(tenant_svc_uri: application[:settings].tenant_uri)
    register(:shard_lookup, sa_client)

    uri = target[:settings].authorization_uri
    target[:logger].debug("privilege_escalation_checker URI: #{uri}")
    escalation_client = UserManagement::Clients::PrivilegeEscalationChecker.new(::Platform::Helpers::GraphQL.http(uri))
    register(:privilege_escalation_checker, escalation_client)

    %i[role_management].each do |service_type|
      setting_name = "#{service_type}_uri".to_sym
      uri = target[:settings].send(setting_name)
      target[:logger].debug("#{service_type} URI: #{uri}")

      unless uri && !uri.empty?
        raise "Service #{service_type} unconfigured: check setting value #{setting_name}"
      end

      client_class = ::UserManagement::Clients.const_get(ActiveSupport::Inflector.camelize(service_type))
      client = client_class.new(
        ::Platform::Helpers::GraphQL.http(uri)
      )
      register(service_type, client)
    end

    # Authorization is initialized outside of the loop above because its
    # client class is in cht-lib-fabric as opposed to the ::UserManagement
    # namespace
    uri = target[:settings].authorization_uri
    target[:logger].debug("authorization URI: #{uri}")
    unless uri && !uri.empty?
      raise "Authorization service unconfigured: check setting value authorization_uri"
    end

    client = Platform::Clients::Authorization.new(::Platform::Helpers::GraphQL.http(uri))
    register(:authorization, client)

  end
end
