# that's defined, or if not, uses a no-op implementation.
#
# This both sets the global OpenTracing tracer object and also
# installs in the dry-system container under the key :tracer.
Application.boot(:tracer) do |application|
    init do
      require 'opentracing'
      require 'zipkin/tracer'

      # This monkey patches
      # https://github.com/salemove/zipkin-ruby-opentracing/blob/master/lib/zipkin/span.rb
      # The Zipkin::Span#initialize method calls Hash#each with a 2-arg
      # method, but that doesn't seem to actually work in any version of
      # Ruby.  Dunno.
      module Zipkin
        class Span
          alias_method :set_tag_orig, :set_tag
          def set_tag(*args)
            if args.length == 1
              set_tag_orig(*args[0])
            else
              set_tag_orig(*args)
            end
          end
        end
      end
    end

    start do
    settings = application[:settings]
      url = if !settings.zipkin_url.empty?
            then settings.zipkin_url
            elsif !settings.wf_host.empty?
            then "http://#{settings.wf_host}:#{settings.zipkin_port}"
            else nil
            end
      ns = settings.k8s_ns
      OpenTracing.global_tracer =
        if url
        then Zipkin::Tracer.build(
               url: url,
               # TODO: service_name needs to match Istio-generated spans
               # (this is fine for prod but doesn't match for PRs or for
               # things that aren't the default web-server component)
               service_name: "user-management.#{ns}"
             )
        else OpenTracing::Tracer.new # nop tracer
        end
      register(:tracer, OpenTracing.global_tracer)
    end
  end