# frozen_string_literal: true

require 'dry/transaction'
require 'dry/system/container'
require 'dry/system/components'
require 'cht-lib-fabric/components'
require 'cht-lib-fabric/dry_transaction_step_adapters'

class Application < ::Dry::System::Container
  use(:env, inferrer: -> { (ENV['RACK_ENV'] || 'development').to_sym })

  configure do |config|
    config.name = SERVICE_NAME
    config.auto_register = %w[app]
  end
  load_paths!('app')

  %i[alert graphql i18n logger].each { |c| boot(c, from: :platform) }
  boot(:statsd, from: :platform) do
    configure do |config|
      config.namespace = "#{SERVICE_NAME}.#{SERVICE_TASK}"
    end
  end

  # If you use Kafka, uncomment the following:
  # boot(:karafka, from: :platform) do
  #   configure do |config|
  #     config.client_id = "#{SERVICE_NAME}-#{SERVICE_TASK}"
  #   end
  # end
  Transaction = ::Dry::Transaction(container: Application, step_adapters: ::Platform::DryTransactionStepAdapters)
end
